@@ -15,29 +15,30 @@
 #  License along with OctoBot. If not, see <https://www.gnu.org/licenses/>.
 import pytest
 
-from exchanges_tests import abstract_authenticated_future_exchange_tester
+from exchanges_tests import abstract_authenticated_exchange_tester
 
 # All test coroutines will be treated as marked.
 pytestmark = pytest.mark.asyncio
 
 
 class TestBybitAuthenticatedExchange(
-    abstract_authenticated_future_exchange_tester.AbstractAuthenticatedFutureExchangeTester
+    abstract_authenticated_exchange_tester.AbstractAuthenticatedExchangeTester
 ):
     # enter exchange name as a class variable here
     EXCHANGE_NAME = "bybit"
     ORDER_CURRENCY = "BTC"
     SETTLEMENT_CURRENCY = "USDT"
-    SYMBOL = f"{ORDER_CURRENCY}/{SETTLEMENT_CURRENCY}:{SETTLEMENT_CURRENCY}"
+    SYMBOL = f"{ORDER_CURRENCY}/{SETTLEMENT_CURRENCY}"
     ORDER_SIZE = 10  # % of portfolio to include in test orders
-    OPEN_ORDERS_IN_CLOSED_ORDERS = True
+    OPEN_TIMEOUT = 20    # larger for bybit testnet
+    CANCEL_TIMEOUT = 20  # larger for bybit testnet
+    CONVERTS_ORDER_SIZE_BEFORE_PUSHING_TO_EXCHANGES = True
+    EXPECT_MISSING_ORDER_FEES_DUE_TO_ORDERS_TOO_OLD_FOR_RECENT_TRADES = True   # when recent trades are limited and
+    # closed orders fees are taken from recent trades
 
     async def test_get_portfolio(self):
         await super().test_get_portfolio()
 
-    async def test_get_empty_linear_and_inverse_positions(self):
-        await super().test_get_empty_linear_and_inverse_positions()
-
     async def test_create_and_cancel_limit_orders(self):
         await super().test_create_and_cancel_limit_orders()
 
@@ -56,12 +57,16 @@ class TestBybitAuthenticatedExchange(
 
     async def test_edit_limit_order(self):
         # pass if not implemented
-        await super().test_edit_limit_order()
+        pass
 
     async def test_edit_stop_order(self):
         # pass if not implemented
-        await super().test_edit_stop_order()
+        pass
 
-    async def test_create_bundled_orders(self):
+    async def test_create_single_bundled_orders(self):
         # pass if not implemented
-        await super().test_create_bundled_orders()
+        pass
+
+    async def test_create_double_bundled_orders(self):
+        # pass if not implemented
+        pass
